import boto3
import argparse

from boto3.dynamodb.conditions import Attr

SCOPED_PREFIX = 'psc:permission:rel:'
OLD_SCOPED = 'psc:permission:rel'

def main():
    dest_env_choices = ['localdev', 'dev08', 'dev01',  'test03', 'eap01', 'prod01', 'prod02', 'prod04', 'prod05', 'prod06', 'prodnrt', 'prodsyd']
    # dest_env_choices = ['localdev', 'hub02', 'hub04', 'hub-prod-us', 'hub-prod-eu', 'hub-prod-ap', 'hub-prod-ap2']

    migration_parser = argparse.ArgumentParser(description="Migrate scoped attribute for Role Permissions from psc:permission:rel to psc:permission:rel:<role_urn>")
    migration_parser.add_argument('-e', '--env', dest='env', required=False, choices=dest_env_choices, help="The enrivonment to run the migration against")
    migration_parser.add_argument('-m', '--migration', action='store_true', dest='migrate', default=False, help="Don't perform the dry run, and perform the actual migration instead.")
    migration_parser.add_argument('-p', '--port', dest='port', default='4569', help="If you want to run this locally and not through docker, you can define the exposed access-mockaws port.")
    migration_parser.add_argument('-q', '--quiet', action='store_true', dest='quiet', default=False, help="Quiet the logs a bit.")
    args = migration_parser.parse_args()
    hub = get_hub(args.env)

    perform_migration = args.migrate
    dry_run = True
    if perform_migration is True:
        dry_run = False

    port = args.port
    quiet = args.quiet

    roles_table_name = '{}_access_roles'.format(hub)
    dynamodb = create_dynamodb_resource(hub, port)
    roles_table = dynamodb.Table(roles_table_name)

    if dry_run is True:
        print('\n** DRY RUN BEGIN')
    else:
        print('\n** DATA MIGRATION BEGIN')

    print('\n-- Querying Role Permissions from DynamoDB in {} on {}'.format(hub, roles_table_name))
    role_permissions = query_role_permissions_with_old_scoped(roles_table)

    print('\n-- Pre-Migration Stats')
    print('Items with scoped = psc:permission:rel')
    print('Count: ', len(role_permissions))

    print('\n-- Performing Migration')
    migrate_role_permissions(role_permissions, roles_table, dry_run, quiet)

    print('\n-- Migration Complete (Still need to verify)')

    if dry_run is True:
        print('\n-- Not performing verification stage since this is a DRY RUN')

    if dry_run is False:
        print('\n-- Querying Role Permissions from DynamoDB in {} on {} using psc:permission:rel to verify count is 0'.format(hub, roles_table_name))
        role_permissions_post_migration = query_role_permissions_with_old_scoped(roles_table)

        if len(role_permissions_post_migration) > 0:
            print('\n-- {} Role Permissions failed to migrate. Run the migration again.'.format(len(role_permissions_post_migration)))
            for rp in role_permissions_post_migration:
                print(rp)

            raise Exception("Some role permission(s) failed to migrate. Run the migration again.")

        if len(role_permissions_post_migration) == 0:
            print('\n-- Migration appears to have been successful. There are zero items with scoped = psc:permission:rel')

        print('\n-- Verifying Role Permissions Post Migration')
        invalid_role_permissions = verify_and_get_invalid_role_permissions(roles_table)

        if len(invalid_role_permissions) > 0:
           print('\n-- {} role permissions have invalid scoped values'.format(len(invalid_role_permissions)))
           for irp in invalid_role_permissions:
               print(irp)

        if len(invalid_role_permissions) == 0:
            print('\n-- Verification returned no invalid role permission')

        print('\n-- Migration Successful')


def verify_and_get_invalid_role_permissions(roles_table):
    role_permissions = query_role_permissions_with_new_scoped(roles_table)

    invalid_role_permissions = []

    for rp in role_permissions:
        if rp['scoped'] != '{}{}'.format(SCOPED_PREFIX, rp['urn']):
            invalid_role_permissions.append(rp)

    return invalid_role_permissions


def create_dynamodb_resource(hub, port):
    if hub == 'localdev':
        dynamodb_client = boto3.resource('dynamodb', region_name='us-east-1', endpoint_url='https://access-mockaws:{}'.format(port), verify=False)
    else:
        dynamodb_client = boto3.resource('dynamodb')
    return dynamodb_client


def query_role_permissions_with_new_scoped(roles_table):
    kwargs = {
        'FilterExpression':Attr('scoped').begins_with(SCOPED_PREFIX)
    }

    return query_role_permissions(roles_table, kwargs)


def query_role_permissions_with_old_scoped(roles_table):
    kwargs = {
        'FilterExpression':Attr('scoped').eq(OLD_SCOPED)
    }

    return query_role_permissions(roles_table, kwargs)


def query_role_permissions(roles_table, kwargs):
    role_permissions = []

    i = 1
    while True:
        print('\n-- Scan page {} of results {}'.format(i, kwargs['ExclusiveStartKey'] if 'ExclusiveStartKey' in kwargs else ''))

        response = roles_table.scan(**kwargs)

        role_permissions += response['Items']

        if 'LastEvaluatedKey' in response:
            i = i + 1
            kwargs['ExclusiveStartKey'] = response['LastEvaluatedKey']
        else:
            break

    return role_permissions

def migrate_role_permissions(role_permissions, roles_table, dry_run, quiet):
    with roles_table.batch_writer() as batch:
        for rp in role_permissions:
            rp['scoped'] = '{}{}'.format(SCOPED_PREFIX, rp['urn'])

            if dry_run is True:
                if quiet is False:
                    print('Would update scoped for role {} | {} | {}'.format(rp['urn'], rp['child_urn'], rp['actions']))

            if dry_run is False:
                if quiet is False:
                    print('Updating scoped for role {} | {} | {}'.format(rp['urn'], rp['child_urn'], rp['actions']))

                batch.put_item(
                    Item=rp
                )

def get_hub(environment):
    hub = None
    localdev_hub = ['localdev']
    dev_hub = ['dev01']
    test_hub = ['test03']
    prod_us_hub = ['eap01', 'prod01', 'prod02', 'prod04', 'prod05']
    prod_eu_hub = ['prod06']
    prod_ap_hub = ['prodnrt']
    prod_ap2_hub = ['prodsyd']

    if environment in localdev_hub:
        hub = 'localdev'
    elif environment in dev_hub:
        hub = 'hub-dev-us'
    elif environment in test_hub:
        hub = 'hub04'
    elif environment in prod_us_hub:
        hub = 'hub-prod-us'
    elif environment in prod_eu_hub:
        hub = 'hub-prod-eu'
    elif environment in prod_ap_hub:
        hub = 'hub-prod-ap'
    elif environment in prod_ap2_hub:
        hub = 'hub-prod-ap2'

    if hub == None:
        raise Exception("Could not get Hub configuration for environment {}".format(environment))
    print("Hub Environment %s " % hub)
    return hub

if __name__ == "__main__":
    main()
