import boto3
import requests
import json
import urllib3
import logging
import argparse
import yaml
import os
import clients as app_clients

from botocore.exceptions import ClientError

urllib3.disable_warnings()
logger = logging.getLogger("remove_secops_role_manager_role_permission")
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s %(levelname)s - %(message)s")
handler_console = logging.StreamHandler()
handler_console.setFormatter(formatter)
logger.addHandler(handler_console)
secops_role_manager_name = 'SECOPS_ROLE_MANAGER'
def main(environment=None):
    CURRENT_FILE_PATH = os.path.dirname(os.path.abspath(__file__))
    environment_choices = ['localdev', 'dev08', 'dev01', 'test03', 'eap01', 'prod01', 'prod02', 'prod04', 'prod05', 'prod06', 'prodnrt', 'prodsyd']
    if environment is not None:
        if environment not in environment_choices:
            raise ValueError('Incorrect environment choice')
    parser = argparse.ArgumentParser(description="Migrate Secops Role Manager")
    parser.add_argument('-e', '--environment', required=False,
                        choices=environment_choices)
    parser.add_argument("-g", "--green-only", help="Run migration hitting only green service", action="store_true")

    args = parser.parse_known_args()[0]

    env = args.environment if environment is None else environment
    if env not in environment_choices:
        raise ValueError('Incorrect environment choice')
    env_urls = yaml.load(open("{}/url_configs.yml".format(CURRENT_FILE_PATH)), Loader=yaml.BaseLoader)[env]
    es_url = env_urls['elasticsearch']

    dynamo_client = app_clients.DynamoDbClient(env)
    dynamo_client.setClientEnvironment()
    es_client = app_clients.EsClient(es_url)
    es_client.setData()

    hub = get_hub(env)
    dynamo_roles_table = '{}_access_roles'.format(hub)
    query_response = es_client.queryConnector(secops_role_manager_name.lower()).json()
    if query_response['hits']['hits'] != []:
        if query_response['hits']['hits'][0]['_source']['name'] == secops_role_manager_name:
            logger.info("Found Secops Role Manager")
            change_manager_permission_request = get_role_manager_role_permission(dynamo_client, hub)
            try:
                dynamo_client.put_item(dynamo_roles_table, change_manager_permission_request)
            except ClientError as ex:
                logger.error("Put item failed for role manager, error: %s)" % ex.response)
            logger.info("Updated Secops Role Manager csr.access.roles permission to only EXECUTE")
    else:
        logger.info("Could not find Secops Role Manager")


def get_role_manager_role_permission(dynamo_client, hub):
    # Current Secops Role Manager csr.access.roles permission
    access_roles_permission = dynamo_client.get_item("{}_access_roles".format(hub),
                                                     {"urn":{"S":"psc:role::SECOPS_ROLE_MANAGER"},
                                                      "child_urn":{"S":"psc:permission:csr.access.roles"}})
    access_roles_permission['Item']['actions']['SS'] = ['EXECUTE']
    return access_roles_permission['Item']

def get_hub(environment):
    hub = None
    localdev_hub = ['localdev']
    dev_hub = ['dev01']
    test_hub = ['test03']
    prod_us_hub = ['eap01', 'prod01', 'prod02', 'prod04', 'prod05']
    prod_eu_hub = ['prod06']
    prod_ap_hub = ['prodnrt']
    prod_ap2_hub = ['prodsyd']

    if environment in localdev_hub:
        hub = 'localdev'
    elif environment in dev_hub:
        hub = 'hub-dev-us'
    elif environment in test_hub:
        hub = 'hub04'
    elif environment in prod_us_hub:
        hub = 'hub-prod-us'
    elif environment in prod_eu_hub:
        hub = 'hub-prod-eu'
    elif environment in prod_ap_hub:
        hub = 'hub-prod-ap'
    elif environment in prod_ap2_hub:
        hub = 'hub-prod-ap2'

    if hub == None:
        raise Exception("Could not get Hub configuration for environment {}".format(environment))
    logger.info("Hub Environment %s " % hub)
    return hub

if __name__ == "__main__":
    main()